@@ -52,6 +52,11 @@ static GObject *gst_time_tool_constructor (GType                  type,
 					   guint                  n_construct_properties,
 					   GObjectConstructParam *construct_params);
 static void  gst_time_tool_update_gui     (GstTool *tool);
+static void  gst_time_tool_update_config  (GstTool *tool);
+
+static void  on_ntp_use_toggled           (GtkWidget *widget,
+					   gpointer   data);
+
 
 G_DEFINE_TYPE (GstTimeTool, gst_time_tool, GST_TYPE_TOOL);
 
@@ -64,6 +69,7 @@ gst_time_tool_class_init (GstTimeToolClass *class)
 	object_class->constructor = gst_time_tool_constructor;
 	object_class->finalize = gst_time_tool_finalize;
 	tool_class->update_gui = gst_time_tool_update_gui;
+	tool_class->update_config = gst_time_tool_update_config;
 
 	g_type_class_add_private (object_class,
 				  sizeof (GstTimeToolPrivate));
@@ -77,16 +83,16 @@ get_ntp_service (GstTimeTool *tool)
 	OobsList *list;
 	OobsListIter iter;
 	gboolean valid;
-	const gchar *role;
+	GstServiceRole role;
 
 	list = oobs_services_config_get_services (tool->services_config);
 	valid = oobs_list_get_iter_first (list, &iter);
 
 	while (valid) {
 		service = oobs_list_get (list, &iter);
-		role = oobs_service_get_role (OOBS_SERVICE (service));
+		role = gst_service_get_role (OOBS_SERVICE (service));
 
-		if (strcmp (role, "NTP_SERVER") == 0)
+		if (role == GST_ROLE_NTP_SERVER)
 			tool->ntp_service = g_object_ref (service);
 
 		g_object_unref (service);
@@ -101,6 +107,7 @@ gst_time_tool_init (GstTimeTool *tool)
 	GstTimeToolPrivate *priv = GST_TIME_TOOL_GET_PRIVATE (tool);
 
 	priv->bus_connection = dbus_bus_get (DBUS_BUS_SESSION, NULL);
+	priv->cookie = 0;
 
 	tool->time_config = oobs_time_config_get (GST_TOOL (tool)->session);
 	tool->ntp_config = oobs_ntp_config_get (GST_TOOL (tool)->session);
@@ -117,12 +124,12 @@ inhibit_screensaver (GstTimeTool *tool,
 	DBusMessage *message, *reply;
 	DBusMessageIter iter;
 
-	g_return_if_fail ((inhibit && priv->cookie == 0) || (!inhibit && priv->cookie != 0));
-
 	if (inhibit) {
 		const gchar *appname = "Time-admin";
 		const gchar *reason = "Changing time";
 
+		g_return_if_fail (priv->cookie == 0);
+
 		message = dbus_message_new_method_call (SCREENSAVER_SERVICE,
 							SCREENSAVER_PATH,
 							SCREENSAVER_INTERFACE,
@@ -134,13 +141,16 @@ inhibit_screensaver (GstTimeTool *tool,
 
 		reply = dbus_connection_send_with_reply_and_block (priv->bus_connection, message, -1, NULL);
 
-		/* get cookie */
-		dbus_message_iter_init (reply, &iter);
-		dbus_message_iter_get_basic (&iter, &priv->cookie);
+		if (reply) {
+			/* get cookie */
+			dbus_message_iter_init (reply, &iter);
+			dbus_message_iter_get_basic (&iter, &priv->cookie);
 
-		dbus_message_unref (message);
-		dbus_message_unref (reply);
-	} else {
+			dbus_message_unref (message);
+			dbus_message_unref (reply);
+		}
+	} else if (!inhibit && priv->cookie != 0) {
+		
 		message = dbus_message_new_method_call (SCREENSAVER_SERVICE,
 							SCREENSAVER_PATH,
 							SCREENSAVER_INTERFACE,
@@ -333,6 +343,35 @@ init_timezone (GstTimeTool *time_tool)
 	time_tool->timezone_dialog = gst_dialog_get_widget (tool->main_dialog, "time_zone_window");
 }
 
+static gboolean
+check_ntp_support (GstTool  *tool,
+		   gboolean  active)
+{
+	GtkWidget *message, *widget;
+
+	if (GST_TIME_TOOL (tool)->ntp_service)
+		return TRUE;
+
+	widget = gst_dialog_get_widget (tool->main_dialog, "ntp_use");
+	g_signal_handlers_block_by_func (widget, on_ntp_use_toggled, tool);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+	g_signal_handlers_unblock_by_func (widget, on_ntp_use_toggled, tool);
+
+	message = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_INFO,
+					  GTK_BUTTONS_CLOSE,
+					  _("NTP support is not installed"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
+						  _("Please install and activate NTP support in the system to enable "
+						    "synchronization of your local time server with "
+						    "internet time servers."));
+	gtk_dialog_run (GTK_DIALOG (message));
+	gtk_widget_destroy (message);
+
+	return FALSE;
+}
+
 static void
 on_ntp_use_toggled (GtkWidget *widget,
 		    gpointer   data)
@@ -343,35 +382,26 @@ on_ntp_use_toggled (GtkWidget *widget,
 	tool = (GstTool *) data;
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-	if (!GST_TIME_TOOL (tool)->ntp_service && active) {
-		GtkWidget *message;
+	if (check_ntp_support (tool, active)) {
+		oobs_service_set_runlevel_configuration (GST_TIME_TOOL (tool)->ntp_service,
+							 oobs_services_config_get_default_runlevel (GST_TIME_TOOL (tool)->services_config),
+							 (active) ? OOBS_SERVICE_START : OOBS_SERVICE_STOP,
+							 /* FIXME: hardcoded priority? */
+							 50);
 
-		g_signal_handlers_block_by_func (widget, on_ntp_use_toggled, tool);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-		g_signal_handlers_unblock_by_func (widget, on_ntp_use_toggled, tool);
-
-		message = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
-						  GTK_DIALOG_MODAL,
-						  GTK_MESSAGE_INFO,
-						  GTK_BUTTONS_CLOSE,
-						  _("NTP support is not installed"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-							  _("Please install and activate NTP support in the system to enable "
-							    "synchronization of your local time server with "
-							    "internet time servers."));
-		gtk_dialog_run (GTK_DIALOG (message));
-		gtk_widget_destroy (message);
-		return;
+		oobs_object_commit (GST_TIME_TOOL (tool)->services_config);
+		gtk_widget_set_sensitive (gst_dialog_get_widget (tool->main_dialog, "timeserver_button"), active);
+		gtk_widget_set_sensitive (GST_TIME_TOOL (tool)->synchronize_now, !active);
 	}
+}
 
-	oobs_service_set_runlevel_configuration (GST_TIME_TOOL (tool)->ntp_service,
-						 oobs_services_config_get_default_runlevel (GST_TIME_TOOL (tool)->services_config),
-						 (active) ? OOBS_SERVICE_START : OOBS_SERVICE_STOP,
-						 /* FIXME: hardcoded priority? */
-						 50);
+static void
+on_synchronize_now_clicked (GtkWidget *widget, gpointer data)
+{
+	GstTimeTool *tool;
 
-	oobs_object_commit (GST_TIME_TOOL (tool)->services_config);
-	gtk_widget_set_sensitive (gst_dialog_get_widget (tool->main_dialog, "timeserver_button"), active);
+	tool = GST_TIME_TOOL (data);
+	gst_tool_commit_async (tool, tool->ntp_config, _("Synchronizing system clock"));
 }
 
 static GObject*
@@ -402,6 +432,10 @@ gst_time_tool_constructor (GType                  type,
 	time_tool->ntp_list = ntp_servers_list_get (time_tool);
 	init_timezone (time_tool);
 
+	time_tool->synchronize_now = gst_dialog_get_widget (GST_TOOL (time_tool)->main_dialog, "synchronize_now_button");
+	g_signal_connect (G_OBJECT (time_tool->synchronize_now), "clicked",
+			  G_CALLBACK (on_synchronize_now_clicked), time_tool);
+
 	return object;
 }
 
@@ -469,9 +503,23 @@ gst_time_tool_update_gui (GstTool *tool)
 	g_signal_handlers_block_by_func (ntp_use, on_ntp_use_toggled, tool);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ntp_use), is_active);
 	gtk_widget_set_sensitive (timeserver_button, is_active);
+	gtk_widget_set_sensitive (time_tool->synchronize_now, !is_active);
 	g_signal_handlers_unblock_by_func (ntp_use, on_ntp_use_toggled, tool);
 }
 
+static void
+gst_time_tool_update_config (GstTool *tool)
+{
+	GstTimeTool *time_tool;
+
+	time_tool = GST_TIME_TOOL (tool);
+	oobs_object_update (time_tool->time_config);
+	oobs_object_update (time_tool->ntp_config);
+	oobs_object_update (time_tool->services_config);
+
+	get_ntp_service (tool);
+}
+
 GstTool*
 gst_time_tool_new (void)
 {
